CREATE OR REPLACE PROCEDURE INSUMOS_FAT_REAL_COC(
P_CDFILIAL IN VARCHAR2,
P_DTINI IN DATE,
P_DTFIN IN DATE,
P_CIAS IN VARCHAR2,
P_DATACUSTO IN DATE,
P_TIPOCUSTO IN NUMBER,
P_TIPORETIRADA IN VARCHAR2,
P_DIAS_RETI IN NUMBER, /* dias anteriores ao perodo informado, para a retirada */
P_NRPRODUCAOCOC IN OUT FLOAT
)
AS
   V_NRPREVSVCOC NUMBER;
   V_CDPREVSVCOC VARCHAR2(10);
   V_DSPREVSVCOC VARCHAR2(50);
   V_NRLISTA VARCHAR2(10);   
   V_NRSERVPREVSVCOC VARCHAR2(20);   
BEGIN

   V_NRPREVSVCOC := -1;
   V_CDPREVSVCOC := '_x_x_x_x_x';
   V_DSPREVSVCOC := 'FAT X REAL';

   /*Insere previso temporria*/      
   INSERT INTO PREVSVCOC
   (NRPREVSVCOC, CDFILIAL, CDPREVSVCOC, DTINIPREVSVCOC, 
    DTFINPREVSVCOC, DTGERACAO, DSPREVSVCOC)
   VALUES
   (V_NRPREVSVCOC, P_CDFILIAL, V_CDPREVSVCOC, P_DTINI,
    P_DTFIN, SYSDATE, V_DSPREVSVCOC);
   
   /*Servios faturados para a previso temporria. Inserir como servios de uma previso (temporria), para calcular os produtos*/
   FOR C_SERV IN
   (SELECT S.CDSERVICO, P.CDCIA, P.CDCLASSE, P.CDCICLO, SUM(NVL(S.QTREALPROD, 0)) AS QTSERV
        FROM DELIVERY D
          INNER JOIN SERVPLANO S ON D.CDFILIAL = S.CDFILIAL AND
                                    D.NRDELIVERY = S.NRDELIVERY
          INNER JOIN PRODUTO P ON P.CDPRODUTO = S.CDSERVICO       
    WHERE D.CDFILIAL = P_CDFILIAL
      AND D.DTEMISDEL BETWEEN P_DTINI AND P_DTFIN
      AND S.NRLANCTONF IS NOT NULL
      AND D.IDSITDEL = '2'
    GROUP BY S.CDSERVICO, P.CDCIA, P.CDCLASSE, P.CDCICLO
   ) LOOP
      
      NOVO_CODIGO_SC('SERVPREVSVCOC',V_NRSERVPREVSVCOC);
      
      INSERT INTO SERVPREVSVCOC
      (NRSERVPREVSVCOC, NRPREVSVCOC, CDSERVICO,
       CDCIA, CDCLASSE, CDCICLO, QTSERVPREVSVCOC)
      VALUES
      (TO_NUMBER(V_NRSERVPREVSVCOC), V_NRPREVSVCOC, C_SERV.CDSERVICO,
       C_SERV.CDCIA, C_SERV.CDCLASSE, C_SERV.CDCICLO, C_SERV.QTSERV);
   END LOOP;

   /*Calcula os produtos dessa previso temporria*/
   PREVISAO_SERVICO_COC( V_NRPREVSVCOC );

   /* Monta lista temporria de previses*/
   V_NRLISTA := '-1';
   INSERT INTO LISTA
   (NRLISTA, CDLISTA)
   VALUES
   (V_NRLISTA, TO_CHAR(V_NRPREVSVCOC));

   /*Calcula os insumos totais da previso temporria (com os custos)*/
   COMPRAS_PREVSVCOC(V_NRLISTA, P_CIAS, 'T', 'T', 
                     P_DATACUSTO, P_TIPOCUSTO, P_NRPRODUCAOCOC);

   /*Quantidade retirada do estoque no perodo para a produo do produto*/
   FOR C_RETI IN
   (
   SELECT I.CDPRODUTO, SUM(NVL(I.QTLANCTOEST, 0)) AS QUANTIDADE
     FROM ITLANCTOEST I
          INNER JOIN PRODPRODUCOC P ON P.CDPRODUTO = I.CDPRODUTO
    WHERE I.CDFILIMOVI = P_CDFILIAL
      AND I.DTLANCMOVI BETWEEN (P_DTINI - P_DIAS_RETI) AND (P_DTFIN - P_DIAS_RETI)
      AND I.IDTIPOMOVI = '3' -- Retirada
      AND I.CDTIPORETI = P_TIPORETIRADA
      AND P.NRPRODUCAOCOC = P_NRPRODUCAOCOC
    GROUP BY I.CDPRODUTO
   ) LOOP
      UPDATE PRODPRODUCOC 
         SET QTRETIRADAPRODUCAO = C_RETI.QUANTIDADE
       WHERE NRPRODUCAOCOC = P_NRPRODUCAOCOC
         AND CDPRODUTO = C_RETI.CDPRODUTO;
   END LOOP;
   
    /*------------------------------------------------*/
    /* Apaga dados temporrios criados                */
    /*------------------------------------------------*/
    DELETE FROM PRODPREVSVCOC WHERE NRPREVSVCOC = V_NRPREVSVCOC;
    DELETE FROM RECPREVSVCOC WHERE NRPREVSVCOC = V_NRPREVSVCOC;
    DELETE FROM SUBGRUPREVSVCOC WHERE NRPREVSVCOC = V_NRPREVSVCOC;
    DELETE FROM SERVPREVSVCOC WHERE NRPREVSVCOC = V_NRPREVSVCOC;
    DELETE FROM PREVSVCOC WHERE NRPREVSVCOC = V_NRPREVSVCOC;
    
    DELETE FROM LISTA WHERE NRLISTA = V_NRLISTA;
END;
/

-- VERSAO --
BEGIN ATUVEROBJ('INSUMOS_FAT_REAL_COC','5.85.xxx'); END;
/